#include "ps2quaternion.h"

using namespace KazMaths;

Quaternion::Quaternion(void)
{
}

Quaternion::Quaternion(float _x, float _y, float _z, float _w)
:w(_w),x(_x),y(_y),z(_z)
{
}

Quaternion::~Quaternion(void)
{
}

Quaternion & Quaternion::NormaliseSelf()
{
	float s = 1.0f / Sqrt(w * w + x * x + y * y + z * z);
	w *= s;
	x *= s;
	y *= s;
	z *= s;

	return *this;
}

Matrix3x3 Quaternion::ToRotationMatrix()
{
	float	xx = 2 * x * x,
			xy = 2 * x * y,
			xz = 2 * x * z,
			yy = 2 * y * y,
			yz = 2 * y * z,
			zz = 2 * z * z,
			wx = 2 * w * x,
			wy = 2 * w * y,
			wz = 2 * w * z;

	return Matrix3x3(1 - yy - zz,	xy + wz,		xz - wy,
			   		 xy - wz,		1 - xx - zz,	yz + wx,					
					 xz + wy,		yz - wx,		1 - xx - yy);
}

Matrix4x4 Quaternion::ToRotationMatrix4x4()
{
	float	xx = 2 * x * x,
			xy = 2 * x * y,
			xz = 2 * x * z,
			yy = 2 * y * y,
			yz = 2 * y * z,
			zz = 2 * z * z,
			wx = 2 * w * x,
			wy = 2 * w * y,
			wz = 2 * w * z;

	return Matrix4x4(1 - yy - zz,	xy + wz,		xz - wy,		0,
			   		 xy - wz,		1 - xx - zz,	yz + wx,		0,					
					 xz + wy,		yz - wx,		1 - xx - yy,	0,					
					 0,				0,				0,				1);
}

void Quaternion::EulerAngles(float roll, float pitch, float yaw)
{
	float cr, cp, cy, sr, sp, sy, cpcy, spsy;

	cr = Cos(roll/2);
	cp = Cos(pitch/2);
	cy = Cos(yaw/2);

	sr = Sin(roll/2);
	sp = Sin(pitch/2);
	sy = Sin(yaw/2);
	
	cpcy = cp * cy;
	spsy = sp * sy;

	x = sr * cpcy - cr * spsy;
	y = cr * sp * cy + sr * cp * sy;
	z = cr * cp * sy - sr * sp * cy;
	w = cr * cpcy + sr * spsy;
}